from jinja2 import Template
from flask import request, Flask

DEFAULT_TEMPLATE = """
<html>
<body>
<h1>{{title}}</h1>
<form method="POST" action="/">
<dl>
{% for field in form %}
    <dt>{{ field.label }}</dt><dd>{{ field() }}</dd>
{% endfor %}
</dl>
    <input type="submit" value="Go">
</form>
{% if result %}
<hr/>
{{ result|safe }}
{% endif %}
</body>
</html>"""

def run_server(action, template=DEFAULT_TEMPLATE, **kargs):

    if isinstance(template, str):
        template = Template(template)
    
    app = Flask(action.__name__)
    @app.route('/', methods=['GET'])
    def get():
        form = action.get_form_class()()
        title = action.__name__
        return template.render(**locals())

    @app.route('/', methods=['POST'])
    def post():
        form = action.get_form_class()(request.form)
        a = action(form)
        result = a.render_result(a.run())
        title = action.__name__
        return template.render(**locals())

    app.run(**kargs)



from .webserver import run_server

class ActionForm(object):
    form_class = None

    @classmethod
    def run_webserver(cls, **args):
        run_server(cls, **args)
    
    @classmethod
    def get_form_class(cls):
        return cls.form_class

    def __init__(self, form=None, **kargs):
        if form is None:
            form = self.get_form_class()(**kargs)
        self.form = self.validate(form)

    def validate(self, form):
        if not form.validate():
            raise ValidationError(form.errors)
        return form 
        
    def run(self):
        return self._run(**self.form.data)

    def render_result(self, result):
        return str(result)



#!/usr/bin/env python

from distutils.core import setup

setup(
    name="actionform",
    version="0.01",
    description="Define an Action as a django form with a run method (with CLI and Web interface)",
    author="Wouter van Atteveldt",
    author_email="wouter@vanatteveldt.com",
    packages=["actionform"],
    keywords = [],
    classifiers=[
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=[
        "wtforms",
    ]
)
